#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.                                        
 | $Id: edit_config.cgi,v 1.6 2002/09/05 21:03:31 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  # Import our library routines.
  require "ids-lib.pl";

  # Print the standard header.
  &header("IDS Management :: Edit Configuration", 0);

  # Set up our help strings and read our configuration.
  %GCONFIG = &idsReadConfiguration();
  $SNORT_INTERFACE_HELP = &hlink("Device to Monitor", 'ids_SNORT_INTERFACE');
  $HOME_NET_HELP     = &hlink("Internal Network(s)",  'ids_HOME_NET');
  $DNS_SERVERS_HELP  = &hlink("DNS Server(s)",        'ids_DNS_SERVERS');
  $HTTP_SERVERS_HELP = &hlink("Web Server(s)",        'ids_HTTP_SERVERS');

  # Set up network/ip input objects.
  my $HOME_NET_VALUE     = &idsMakeAddressInput('HOME_NET',
                                                $GCONFIG{'HOME_NET'}, 1);
  my $DNS_SERVERS_VALUE  = &idsMakeAddressInput('DNS_SERVERS',
                                                $GCONFIG{'DNS_SERVERS'});
  my $HTTP_SERVERS_VALUE = &idsMakeAddressInput('HTTP_SERVERS',
                                                $GCONFIG{'HTTP_SERVERS'});

  # Print some HTML.
  print <<EOF;
<P><CENTER>
  <FORM ACTION="do_edit_config.cgi" METHOD="POST">
  <TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0" WIDTH="620">
  <TR $tb>
    <TD COLSPAN="2"><DIV CLASS="ttitle"><B>
      IDS Management :: Edit Configuration
    </B></DIV></TD>
  </TR>

  <TR $cb>
    <TD COLSPAN="2"><DIV CLASS="nbody">
      Below you can configure your IDS.
    </DIV></TD>
  </TR>

  <TR $cb> <TD COLSPAN="2"> &nbsp; </TD> </TR>

  <TR $cb>
    <TD WIDTH="10"> &nbsp; </TD>
    <TD>
      <TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0" WIDTH="100%">

      <TR $tb>
        <TD COLSPAN="2"><DIV CLASS="thead">
          <B>IDS Configuration</B>
        </DIV></TD>
      </TR>

      <TR>
        <TD COLSPAN="2"><DIV CLASS="nbody">
          These settings affect the operation of the intrusion detection
          system.
        </DIV></TD>
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR>
        <TD VALIGN="TOP"><DIV CLASS="nbody"><B> $SNORT_INTERFACE_HELP </B></DIV></TD>
        <TD><DIV CLASS="toolfont">
          <SELECT NAME="SNORT_INTERFACE">
            <OPTION VALUE="">-- Please Select --</OPTION>
EOF

  #
  # Walk over all our interfaces showing as we go.  SELECT the one we
  # currently have configured.
  #
  foreach my $i (&idsListInterfaces()) {
    print "            <OPTION VALUE=\"$i->{'iface'}\"";
    print "SELECTED" if ($i->{'iface'} eq $GCONFIG{'SNORT_INTERFACE'});
    print ">$i->{'iface'} ($i->{'addr'})</OPTION>\n";
  }

  # More HTML.
  print <<EOF;
          </OPTION>
        </DIV></TD>
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR>
        <TD VALIGN="TOP"><DIV CLASS="nbody"><B>$HOME_NET_HELP</B></DIV></TD>
        <TD>
$HOME_NET_VALUE
        </TD>
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR>
        <TD VALIGN="TOP"><DIV CLASS="nbody"><B>$DNS_SERVERS_HELP</B></DIV></TD>
        <TD VALIGN="TOP">
$DNS_SERVERS_VALUE
        </TD>
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR>
        <TD VALIGN="TOP"><DIV CLASS="nbody"><B>$HTTP_SERVERS_HELP</B></DIV></TD>
        <TD>
$HTTP_SERVERS_VALUE
        </TD>
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>
      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>


      <!-- "Rule Configuration" -->
      <TR $tb>
        <TD COLSPAN="2"><DIV CLASS="thead">
          <B>Rule Configuration</B>
        </DIV></TD>
      </TR>

      <TR>
        <TD COLSPAN="2"><DIV CLASS="nbody">
          Below are groups of rules for the IDS.  The IDS will use whichever
          ones are checked.
        </DIV
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>
      <TR>
        <TD COLSPAN="2">
        <TABLE BORDER="0" WIDTH="100%" CELLPADDING="0" CELLSPACING="0">
          <TR>
EOF

  #
  # Iterate over all of the rules, checking as we go.
  #
  my $TRACK = 0;
  foreach my $r (glob "/var/chroot/snort/etc/*.rules") {
    my $NAME = `basename "$r" .rules`; chomp $NAME;
    my $tmp_CHECKED = (defined($GCONFIG{"rule_$NAME"})) ? "CHECKED" : "";
    my $hlink = &hlink($NAME, "ids_include_$NAME");

    print <<EOF;
        <TD WIDTH="35%"><DIV CLASS="nbody"><B>$hlink</B></DIV></TD>
        <TD WIDTH="15%">
          <INPUT TYPE="CHECKBOX" NAME="rule_$NAME" VALUE="1" $tmp_CHECKED>
        </TD>
EOF

    $TRACK++; print "</TR>\n<TR>\n" if (($TRACK % 3) == 0);
  }
  print "<TD>&nbsp;</TD>\n<TD>&nbsp;</TD>" if (($TRACK % 3) != 0);

  # More HTML.
  print <<EOF;
          </TR>
        </TABLE>
        </TD>
      </TR>

      <TR> <TD COLSPAN="2"> &nbsp; </TD> </TR>

      <TR>
        <TD COLSPAN="2" ALIGN="RIGHT"><DIV CLASS="nbody">
          <INPUT TYPE="SUBMIT" VALUE="Save Changes">
        </DIV></TD>
      </TR>

      </TABLE>
    </TD>
  </TR>

  </TABLE>
  </FORM>
</CENTER><P>
EOF

  # Print the standard footer.
  &ryanFooter("index.cgi");

################################################################################
sub idsListInterfaces() {
  my @RV;

  open IFCONFIG, '/sbin/ifconfig |' or return @RV;
  while (my $line = <IFCONFIG>) {
    chomp $line;

    if ($line =~ m/^([A-Za-z0-9]+)\s+/) {
      my $IF = $1;

      # Skip certain interface names; get the next line (has IP address).
      next if ($IF eq 'lo' || $IF =~ m/^ipsec/);
      $line = <IFCONFIG>; chomp $line;

      if ($line =~ m/^\s+inet\s+addr\:([0-9\.]+)\s+/) {
        push @RV, { 'iface' => $IF, 'addr' => $1 };
      }
    }
  }
  close IFCONFIG;

  return @RV;
}
