#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2000-2002, Guardian Digital Inc.
 | $Id: mrtg-lib.pl,v 1.6 2002/04/18 18:19:31 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ### Do some initial setup.
  do "../../web-lib.pl";
  &init_config();

  ##############################################################################
  ### Global Variables
  ##############################################################################
  $MRTG_CFG          = '/etc/mrtg/mrtg.cfg';
  $MRTG_WORK_DIR     = '/home/httpd/html/system/mrtg';
  $MRTG_RELETIVE_DIR = '/StatusMonitor/mrtg';
#  $MRTG_COLOR_I      = "<FONT COLOR=\"#7aafff\">";
#  $MRTG_COLOR_O      = "<FONT COLOR=\"#1000ff\">";

#  $MRTG_COLOR_I      = "<FONT COLOR=\"#64768d\">";
  $MRTG_COLOR_I      = "<FONT COLOR=\"#9999CC\">";
  $MRTG_COLOR_O      = "<FONT COLOR=\"#660066\">";

# 
# How about this color combination
# --------------------------------
# Light Blue: #7aafff
# Blue:       #1000ff
# Cool Blue:  #64768d	(OUT)
# Purple:     #660066	(IN)
# 

  @MRTG_GRAPHS = ( 
                   { 'disp'    => 1,
                     'short'   => '%',
                     'legendI' => 'Total',
                     'legendO' => 'User',
                     'name'    => 'cpu',
                     'ditty'   => 'Total Utilization vs User Utilization',
                     'desc'    => 'Processors' },
                     
                   { 'disp'    => 2,
                     'short'   => '&#176;&nbsp;F',
                     'legendI' => 'CPU',
                     'legendO' => 'System',
                     'name'    => 'sensors',
                     'ditty'   => 'CPU Temperature vs System Temperature',
                     'desc'    => 'Temperatures' },

                   { 'disp'    => 3,
                     'short'   => '',
                     'legendI' => 'In',
                     'legendO' => 'Out',
                     'name'    => 'eth0',
                     'ditty'   => 'Inbound Traffic vs Outbound Traffic',
                     'desc'    => 'Ethernet (eth0)' },

                   { 'disp'    => 4,
                     'short'   => '',
                     'legendI' => 'In',
                     'legendO' => 'Out',
                     'name'    => 'eth1',
                     'ditty'   => 'Inbound Traffic vs Outbound Traffic',
                     'desc'    => 'Ethernet (eth1)' },

                   { 'disp'    => 5,
                     'short'   => 'MB',
                     'legendI' => 'Swap',
                     'legendO' => 'RAM',
                     'name'    => 'ram',
                     'ditty'   => 'Swap Used vs RAM Used',
                     'desc'    => 'Memory' }
                 );

 
  ##############################################################################
  ### | mrtgGetStats()
  ### |
  ### | Return a hash populated with daily/weekly/monthly/yearly
  ### | max/min/current in/out values.  Keys are of the form:
  ### |
  ### |   [dwmy]_(max|av|cu)(in|out) (Examples:  d_maxin, w_minout)
  ##############################################################################
  sub mrtgGetStats() {
    my (%RV, $SUFFIX);

    foreach my $G (@MRTG_GRAPHS) {
      if ($G->{'name'} eq $_[0]) { $SUFFIX = "&nbsp;$G->{'short'}"; }
    }

    open FILE, "$MRTG_WORK_DIR/$_[0].html";
      while (my $LINE = <FILE>) {
        chomp $LINE;

        if ($LINE =~ m/^<META\ HTTP\-EQUIV=\"Date\" CONTENT=\"(.*)\">$/) {
          $RV{'date'} = $1;
        }

        if ($LINE =~ m/^<\!\-\-\ ([a-z]+)\ ([dwmy])\ ([0-9]+)\ \-\->$/) {
          my ($KEY, $TMP); $KEY = "$2_$1"; $TMP = $3;

          if ($_[0] eq 'ram')      { $TMP = int($TMP / 1048576); }
          elsif ($_[0] =~ m/^eth/) {
            if ($TMP > 1024) { $TMP = $TMP / 1024; $SUFFIX = "&nbsp;KB/s"; }
            if ($TMP > 1024) { $TMP = $TMP / 1024; $SUFFIX = "&nbsp;MB/s"; }
            if ($TMP > 1024) { $TMP = $TMP / 1024; $SUFFIX = "&nbsp;GB/s"; }

            $TMP = sprintf "%6.1f", $TMP;
          }

          $RV{"$KEY"} = "$TMP$SUFFIX";
        } 

      }
    close FILE;

    return %RV;
  }

  ##############################################################################
  ### | mrtgPrintBlock(DWMY, G, INDEX)
  ### |
  ### | Print a Daily/Weekly/Montly/Yearly graph of the MRTG object in $G.  If
  ### | $INDEX is defined then assume this is on index.cgi and print a more
  ### | broad header.
  ##############################################################################
  sub mrtgPrintBlock() {
    my ($DWMY, $G, %STATS, $LINK_START, $LINK_STOP);
    $DWMY = ($_[0]); $G = $_[1]; %STATS = &mrtgGetStats($G->{'name'});

    # Don't do jack if we don't have a graph.
    next if (! -f "$MRTG_WORK_DIR/$G->{'name'}-day.png");

    # bewm
    if ($DWMY eq 'd')    { $DWMY_LONG = 'Daily Usage'; }
    elsif ($DWMY eq 'w') { $DWMY_LONG = 'Weekly Usage'; }
    elsif ($DWMY eq 'm') { $DWMY_LONG = 'Monthly Usage'; }
    elsif ($DWMY eq 'y') { $DWMY_LONG = 'Yearly Usage'; }

    # This is the index page, do things differently.
    if ($_[2]) {
      $DWMY_LONG  = "$G->{'desc'}:&nbsp;&nbsp;$G->{'ditty'}";
      $LINK_START = "          <A HREF=\"applet_graphs.cgi?name=$G->{'name'}\">";
      $LINK_STOP  = "          </A>";
    }

    print <<EOF;
        <TR $tb> <TD COLSPAN="4"><DIV CLASS="thead"><B>$DWMY_LONG</B></DIV></TD> </TR>
        <TR $cb>
          <TD WIDTH="30"><DIV CLASS="nbody">
            $MRTG_COLOR_I<B>$G->{'legendI'}:</B></FONT>
          </DIV></TD>

          <TD WIDTH="30"><DIV CLASS="nbody">$MRTG_COLOR_I<B>
            $STATS{"$DWMY\_maxin"} (Max.)
          </B></FONT></DIV></TD>
          <TD WIDTH="30"><DIV CLASS="nbody">$MRTG_COLOR_I<B>
            $STATS{"$DWMY\_avin"} (Avg.)
          </B></FONT></DIV></TD>
          <TD WIDTH="30"><DIV CLASS="nbody">$MRTG_COLOR_I<B>
            $STATS{"$DWMY\_cuin"} (Cur.)
          </B></FONT></DIV></TD>
        </TR>
EOF

# $STATS{"$DWMY\_maxin"} / $STATS{"$DWMY\_avin"} / $STATS{"$DWMY\_cuin"}
    if ($G->{'legendI'} ne $G->{'legendO'}) {
      print <<EOF;
        <TR $cb>
          <TD WIDTH="30"><DIV CLASS="nbody">
            $MRTG_COLOR_O<B>$G->{'legendO'}:</B></FONT>
          </DIV></TD>

          <TD WIDTH="30"><DIV CLASS="nbody">$MRTG_COLOR_O<B>
            $STATS{"$DWMY\_maxout"} (Max.)
          </B></FONT></DIV></TD>
          <TD WIDTH="30"><DIV CLASS="nbody">$MRTG_COLOR_O<B>
            $STATS{"$DWMY\_avout"} (Avg.)
          </B></FONT></DIV></TD>
          <TD WIDTH="30"><DIV CLASS="nbody">$MRTG_COLOR_O<B>
            $STATS{"$DWMY\_cuout"} (Cur.)
          </B></FONT></DIV></TD>
        </TR>

EOF
    }
#    if ($G->{'legendI'} ne $G->{'legendO'}) {
#      print <<EOF;
#        <TR $cb>
#          <TD><DIV CLASS="nbody">$MRTG_COLOR_O<B>Max&nbsp;$G->{'legendO'}:</B></DIV></TD>
#          <TD><DIV CLASS="nbody"><B>$STATS{"$DWMY\_maxout"}</B></DIV></TD>
#          <TD><DIV CLASS="nbody">$MRTG_COLOR_O<B>Average&nbsp;$G->{'legendO'}:</B></DIV></TD>
#          <TD><DIV CLASS="nbody"><B>$STATS{"$DWMY\_avout"}</B></DIV></TD>
#          <TD><DIV CLASS="nbody">$MRTG_COLOR_O<B>Current&nbsp;$G->{'legendO'}:</B></DIV></TD>
#          <TD><DIV CLASS="nbody"><B>$STATS{"$DWMY\_cuout"}</B></DIV></TD>
#        </TR>
#EOF
#}

    print <<EOF;
        <TR $cb> <TD COLSPAN="4"> &nbsp; </TD> </TR>

        <TR $cb> <TD COLSPAN="4"><DIV CLASS="nbody">
$LINK_START
          <IMG SRC="/StatusMonitor/mrtg/showimage.cgi?name=$G->{'name'}&dwmy=$DWMY" BORDER="0">
$LINK_STOP
        </TD> </TR>

        <TR $cb> <TD COLSPAN="4"> &nbsp; </TD> </TR>

EOF

  }

  ##############################################################################
  1;
