#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: servicemon-lib.pl,v 1.2 2002/03/18 19:44:33 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ###  Import our system subroutines.
  do "../StatusMonitor.pl";

  ### ##########################################################################
  ### Global Variables
  ### ##########################################################################
  $XINETD_CONFIG_DIR  =  '/etc/xinetd.d';
  $CHKCONFIG_BIN      =  '/sbin/chkconfig';

  @SERVICES = (
                { 'name' => 'FTP Server',
                  'port' => '21',
                  'init' => 'xinetd' },

                { 'name' => 'DHCP Server',
                  'port' => 67,
                  'init' => 'dhcpd' },

                { 'name' => 'Web Server',
                  'port' => 80,
                  'init' => 'httpd' },

                { 'name' => 'Domain Name Server',
                  'port' => 53,
                  'init' => 'named' },

                { 'name' => 'Mail Server',
                  'port' => 25,
                  'init' => 'postfix' },

                { 'name' => 'PPTP Server',
                  'port' => 1723,
                  'init' => 'pptpd' },

                { 'name' => 'Windows File Sharing',
                  'port' => 137,
                  'init' => 'samba' },

                { 'name' => 'SIMAP Server',
                  'port' => 993,
                  'init' => 'stunnel-imap' },

                { 'name' => 'SPOP3 Server',
                  'port' => 995,
                  'init' => 'stunnel-pop3' },

                { 'name' => 'User Password Changer',
                  'port' => 1022,
                  'init' => 'userpass' },

                { 'name' => 'Caching Proxy',
                  'port' => 8080,
                  'init' => 'squid' }
              );


  ### ##########################################################################
  sub isServiceActive() {
    my ($tmp); $tmp = undef;

    $tmp  = `netstat --inet -an | grep :$_[0]`;

    ###  Special case for Postfix.
    if (! $tmp && ($_[0] eq "25")) {
      $tmp = `pidof master`;
      $tmp = undef if ($tmp !~ m/^[0-9]+$/);
    }

    if ($tmp) { return 1; } 
    else      { return 0; }
  }


  ### ##########################################################################
  sub isServiceActiveAtBoot() {
    my ($tmp);

    ($tmp) = (`chkconfig --list $_[0]` =~ m/\s+3\:([onf]+)/);
 
    if ($tmp eq "on") { return 1; } 
    else              { return 0; }
  }


  ### ##########################################################################
  sub toggleRunningService() {
    my ($rc); $rc = undef;

    ###  We run several services out of xinetd, so we need to differentiate.
    if ($_[1] ne "xinetd") {
      my (%E);

      ###  Save our environment, do the action, then restore the environment.
      %E = %ENV; %ENV = ();
        $rc = system "/etc/init.d/$_[0] $_[1] > /dev/null 2>&1";
      %ENV = %E;
    }
    else { }	### Hmm... (bewm)

    return $rc;
  }


  ### ##########################################################################
  sub toggleBootableService() {
    my ($rc); $rc = undef;

    $rc = system "chkconfig", $_[1], $_[0];

    return $rc;
  }


  ### ##########################################################################
  sub toggleXinetdService() {
    my ($lref, $rc);

    $lref = &read_file_lines("$XINETD_CONFIG_DIR/$_[0]");

    ###  Comment out the "disable =" line if we want it enabled.
    for (my $i = 0; $i < @$lref; $i++) {
      if (@$lref[$i] =~ m/^.*disable\s+\=/) {
        if ($_[1] eq "on") { @$lref[$i] = "\t# disable\t\t= yes"; }
        else               { @$lref[$i] = "\tdisable\t\t= yes"; }
      }
    }

    &flush_file_lines();

    ###  All done, so reload the xinetd services.
    my (%E); %E = %ENV; %ENV = ();
      $rc = system "/etc/init.d/xinetd", "restart";
    %ENV = %E;

    return $rc;
  }


  ### ##########################################################################
  sub portsort() { $a->{'port'} <=> $b->{'port'} };


  ### ##########################################################################
  sub PrintError {
    my ($message) = @_;

    print <<ErrorEOF;
Content-type: text/html


<HTML>
<HEAD>
  <TITLE> </TITLE>

  <SCRIPT LANGUAGE="JavaScript">
    alert("$message");
    history.go(-1);
  </SCRIPT>
</HEAD>
</HTML>
ErrorEOF
  }


  ### ##########################################################################
  1;

