#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: systemstats.cgi,v 1.1.1.2 2001/12/11 06:50:39 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ###  Import our library subroutines.
  require './StatusMonitor.pl';


  ###  Define local variables.
  my (%SYSINFO, @FSINFO, @NETINFO, @INTERFACES, %SERVICES);

  ###  Setup local variables (System Information).
  ($SYSINFO{'HOSTNAME'}, $SYSINFO{'KERNELVER'}) = split /\ /, `uname -nr`;
  ($SYSINFO{'UPTIME'},   $SYSINFO{'LOADAVG'})   = (`uptime` =~ m/up\ ([^\,]+).*\: ([0-9\.\,\ ]+)$/);


  ###  Setup local variables (Filesystem Information).
  foreach my $df (`df -h`) {
    my (@f); @f = split /\ +/, $df;

    next if ($f[0] !~ m/^\//);

    push @FSINFO, { 'FS' => $f[0],    'SIZE' => $f[1],  'USED' => $f[2],
                    'AVAIL' => $f[3], 'PUSED' => $f[4], 'MOUNT' => $f[5] };
  }

  ###  Setup local variables (Active Network Interfaces).
  my ($IN_IFACE); $IN_IFACE = 0;
  foreach my $if (`ifconfig`) {

    if ($if =~ m/^([a-z0-9\:]+)\s+/) {
      $IN_IFACE  =  1; $tmp_iface = $1;
    }
    elsif ($if eq "")  { $IN_IFACE = 0; $tmp_iface = ""; }

    if (($if =~ m/^\s+inet\ addr\:([0-9\.]+).*Mask\:([0-9\.]+)$/) &&
        $IN_IFACE) {
      push @INTERFACES, { 'iface'   => $tmp_iface,
                          'address' => $1,
                          'netmask' => $2 }; 
    }
  }


  ###  Setup local variables (Active Network Connections).
  foreach my $net (`netstat --inet -an`) {
    my (@n); @n = split /[\ \:]+/, $net;

    if ($n[0] eq "tcp") { 
      push @NETINFO, { 'PROTO' => 'tcp', 'LADDR' => $n[3], 'LPORT' => $n[4],
                       'FADDR' => $n[5], 'FPORT' => $n[6], 'STATE' => $n[7] };
    }
    elsif ($n[0] eq "udp") {
      push @NETINFO, { 'PROTO' => 'udp', 'LADDR' => $n[3], 'LPORT' => $n[4],
                       'FADDR' => $n[5], 'FPORT' => $n[6], 'STATE' => "&nbsp;"};
    }
    else { next; }

  }


  ###  Print the standard header.
  &header("System Statistics", "");

  ###  Print the HTML.
  print <<EOF;
<P><CENTER>
<TABLE BORDER="0" WIDTH="620" CELLSPACING="0" CELLPADDING="0">
  <TR $tb>
    <TD COLSPAN="2"><DIV CLASS=ttitle><B>System Statistics</B></DIV></TD>
  </TR>

  <TR $cb>
    <TD COLSPAN="2"><DIV CLASS="nbody">
      Below is a system overview.
    </DIV></TD>
  </TR>

  <TR $cb> <TD COLSPAN="2"> &nbsp;</TD> </TR>

  <TR $cb>
    <TD WIDTH="10"> &nbsp </TD>
    <TD>
      <TABLE BORDER="0" WIDTH="100%" CELLSPACING="0" CELLPADDING="0">
      <TR $tb>
        <TD><DIV CLASS="thead"><B>System Information</B></DIV></TD>
      </TR>
      <TR $cb>
        <TD>
          <TABLE BORDER="0" WIDTH="100%" CELLSPACING="0" CELLPADDING="0">
          <TR>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Machine Name</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$SYSINFO{'HOSTNAME'}</DIV></TD>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Uptime</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$SYSINFO{'UPTIME'}</DIV></TD>
          </TR>

          <TR>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Kernel Version</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$SYSINFO{'KERNELVER'}</DIV></TD>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Load Average</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$SYSINFO{'LOADAVG'}</DIV></TD>
          </TR>
          </TABLE>
        </TD>
      </TR>

      <TR $cb> <TD> &nbsp; </TD> </TR>


      <TR $tb>
        <TD><DIV CLASS="thead"><B>Filesystem Information</B></DIV></TD>
      </TR>
      <TR $cb>
        <TD>
          <TABLE BORDER="0" WIDTH="100%" CELLSPACING="0" CELLPADDING="0">
EOF


  ###  Iterate through each FS, printing as we go.
  foreach my $f (@FSINFO) {
    print <<EOF;
          <TR>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Filesystem</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$f->{'FS'}</DIV></TD>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Mount Point</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$f->{'MOUNT'}</DIV></TD>
          </TR>

          <TR>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Size / Used</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$f->{'SIZE'} / $f->{'USED'} ($f->{'PUSED'})</DIV></TD>
            <TD WIDTH="20%"><DIV CLASS="nbody"><B>Space Available</B></DIV></TD>
            <TD WIDTH="30%"><DIV CLASS="nbody">$f->{'AVAIL'}</DIV></TD>
          </TR>

          <TR> <TD COLSPAN="4"> &nbsp; </TD> </TR>

EOF
  }


  ###  More HTML.
  print <<EOF;
          </TABLE>
        </TD>
      </TR>

      <TR $cb> <TD> &nbsp; </TD> </TR>

      <TR $tb>
        <TD><DIV CLASS="thead"><B>Active Network Interfaces</B></DIV></TD>
      </TR>

      <TR $cb>
        <TD>
          <TABLE BORDER="0" WIDTH="100%" CELLSPACING="0" CELLPADDING="0">
EOF


  ###  Iterate through each interface printing as we go. (bewm)
  my ($COUNT); $COUNT = 0;
  foreach my $if (@INTERFACES) {
    $COUNT++;

    if ($COUNT == 1) {
    print <<EOF;
           <TR>
             <TD><DIV CLASS="nbody"><B>Interface</B></DIV></TD>
             <TD><DIV CLASS="nbody"><B>Address / Netmask</B></DIV></TD>
           </TR>
EOF
    }

    my ($I_STR);
    $I_STR = &Make_Popup($if->{'iface'}, "popup_ifconfig.cgi", 500, 150);

    print <<EOF;
           <TR>
             <TD><DIV CLASS="nbody">$I_STR</DIV></TD>
             <TD><DIV CLASS="nbody">$if->{'address'} / $if->{'netmask'}</DIV></TD>
           </TR>
EOF
  }


  ###  More HTML.
  print <<EOF;
          </TABLE>
        </TD>
      </TR>

      <TR $cb> <TD> &nbsp; </TD> </TR>
      <TR $cb> <TD> &nbsp; </TD> </TR>

      <TR $tb>
        <TD><DIV CLASS="thead"><B>Active Network Connections</B></DIV></TD>
      </TR>

      </TR>
      <TR $cb>
        <TD>
          <TABLE BORDER="0" WIDTH="100%" CELLSPACING="0" CELLPADDING="0">
EOF


  ###  Iterate through each active network connection, printing as we go.
  my ($COUNT); $COUNT = 0;
  foreach my $f (@NETINFO) {

    ###  Don't display daemons.
    next if ($f->{'FADDR'} eq "0.0.0.0");
    $COUNT++;

    if ($COUNT == 1) {
      print <<EOF;
          <TR>  <TD COLSPAN="5"> &nbsp; </TD> </TR>

          <TR>
            <TD WIDTH="2%"> &nbsp; </TD>
            <TD WIDTH="8%"  $tb><DIV CLASS="thead"><B>Protocol</B></DIV></TD>
            <TD WIDTH="20%" $tb><DIV CLASS="thead"><B>State</B></DIV></TD>
            <TD WIDTH="35%" $tb><DIV CLASS="thead"><B>Local Address</B></DIV></TD>
            <TD WIDTH="35%" $tb><DIV CLASS="thead"><B>Remote Address</B></DIV></TD>
          </TR>

EOF
    }

    my ($L_STR, $F_STR);
    $L_STR = &Make_Popup($f->{'LADDR'}, "popup_lookup.cgi", 500, 400);
    $F_STR = &Make_Popup($f->{'FADDR'}, "popup_lookup.cgi", 500, 400);

    print <<EOF;
          <TR $cb>
            <TD> &nbsp; </TD>
            <TD><DIV CLASS="nbody">$f->{'PROTO'}</DIV></TD>
            <TD><DIV CLASS="nbody">$f->{'STATE'}</DIV></TD>
            <TD><DIV CLASS="nbody">$L_STR ($f->{'LPORT'})</DIV></TD>
            <TD><DIV CLASS="nbody">$F_STR ($f->{'FPORT'})</DIV></TD>
          </TR>

EOF
  }


  ###  More HTML.
  print <<EOF;
          </TABLE>
        </TD>
      </TR>

      </TR>
      <TR $cb>
        <TD>
          <TABLE BORDER="0" WIDTH="100%" CELLSPACING="0" CELLPADDING="0">
EOF


  ###  Iterate through each active network connection, printing as we go.
  my ($COUNT); $COUNT = 0;
  foreach my $f (sort listensort @NETINFO) {

    ###  Don't display daemons.
    next if ($f->{'FADDR'} ne "0.0.0.0");
    $COUNT++;

    if ($COUNT == 1) {
      print <<EOF;
          <TR>  <TD COLSPAN="5"> &nbsp; </TD> </TR>

          <TR>
            <TD WIDTH="2%"> &nbsp; </TD>
            <TD WIDTH="20%" $tb><DIV CLASS="thead"><B>Service</B></DIV></TD>
            <TD WIDTH="35%" $tb><DIV CLASS="thead"><B>Listen Address</B></DIV></TD>
            <TD> &nbsp; </TD>
          </TR>

EOF
    }

    ###  Get this service name, then print.
    my ($tmp);
    $tmp = getservbyport($f->{'LPORT'}, $f->{'PROTO'});
    $tmp = "&lt;Unknown&gt;" if (! $tmp); 

    print <<EOF;
          <TR $cb>
            <TD> &nbsp; </TD>
            <TD><DIV CLASS="nbody">$tmp</DIV></TD>
            <TD><DIV CLASS="nbody">$f->{'LADDR'}</DIV></TD>
            <TD> &nbsp; </TD>
          </TR>

EOF
  }


  ###  More HTML.
  print <<EOF;
          </TABLE>
        </TD>
      </TR>

      <TR $cb> <TD> &nbsp; </TD> </TR>

      </TABLE>
    </TD>
  </TR>
</TABLE><P>
EOF


  ###  Print the standard footer.
  &ryanFooter("index.cgi");


sub Make_Popup() {
  return "<A onClick='window.open(\"$_[1]?queryinput=$_[0]\", \"help\", \"toolbar=no,menubar=no,scrollbars=yes,width=$_[2],height=$_[3],resizable=yes\"); return false' HREF=\"$_[1]?queryinput=$_[0]\">$_[0]</A>";
}

sub listensort() { $b->{'LADDR'} cmp $a->{'LADDR'} };
