#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.                                        
 | $Id: do_edit_physical_interface.cgi,v 1.2 2002/07/24 14:43:50 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  ###  Import our library subroutines and populate %in.
  require "network-lib.pl";
  &ReadParse();


  ###
  ###  Case 1:  CREATE, STATIC
  ###  
  if ($in{'create'} && ($in{'type'} eq "static")) {
    my ($lref, $network, $broadcast, $if_ID);


    ###  Input Validation
    if (! $in{'address'}) {
      &PrintError("Please enter an 'IP Address'"); exit;
    }
    elsif (! $in{'netmask'}) {
      &PrintError("Please enter a 'Netmask'"); exit;
    }
    elsif (! &isIP($in{'address'})) {
      &PrintError("You entered an invalid 'IP Address'"); exit;
    }
    elsif (! &isIP($in{'netmask'})) {
      &PrintError("You entered an invalid 'Netmask'"); exit;
    }


    ###  Setup local variables.
    $if_ID       =  GetNextPhysicalInterfaceNumber();
    $network     =  &GetNetwork($in{'address'}, $in{'netmask'});
    $broadcast   =  &GetBroadcast($in{'address'}, $in{'netmask'});
    $lref        =  &read_file_lines($INTERFACES_FILE);

      ###  Push our new entry into the file.
      push @$lref, "iface eth$if_ID inet static";
      push @$lref, "   address $in{'address'}";
      push @$lref, "   netmask $in{'netmask'}";
      push @$lref, "   network $network";
      push @$lref, "   broadcast $broadcast\n";

    ###  Finally, write the file to disk and bring the interface up.
    &flush_file_lines();
    &IfUp("eth$if_ID");
  }

  ###
  ###  Case 2:  EDIT/DELETE, STATIC
  ###  
  elsif (($in{'save'} || $in{'delete'}) && ($in{'type'} eq "static")) {
    my ($CURRENT, $lref, $OFFSET, $LENGTH);

    ###  Get $CURRENT.
    foreach my $i (&ReadInterfaces) {
      if ($i->{'interface'} eq $in{'iface'}) { $CURRENT = $i; }
    }

    ###  Setup local variables.
    $lref   =  &read_file_lines($INTERFACES_FILE);
    $OFFSET =  $CURRENT->{'START'};
    $LENGTH =  $CURRENT->{'END'} - $CURRENT->{'START'} + 1;


    ###  Case 1:  We want to delete this interface, so just splice it out.
    if ($in{'delete'}) {
      splice @$lref, $OFFSET, $LENGTH;
      &IfDown($CURRENT->{'interface'});
      &flush_file_lines();
    }
    ###  Case 2:  We want to save this interface, so build up $new and replace.
    else {
      my (@new, $network, $broadcast);

      ###  Recalculate network and broadcast numbers.
      $network     =  &GetNetwork($in{'address'}, $in{'netmask'});
      $broadcast   =  &GetBroadcast($in{'address'}, $in{'netmask'});

      ###  Build up our new interface block.
      push @new, "iface $CURRENT->{'interface'} inet static";
      push @new, "   address $in{'address'}";
      push @new, "   netmask $in{'netmask'}";
      push @new, "   network $network";
      push @new, "   broadcast $broadcast";

        ###  Append the default route line if we have one.
        if ($CURRENT->{'DEFAULTROUTE'}) {
          push @new, "   up route add default gw $CURRENT->{'DEFAULTROUTE'}"
        }

      ###  Finally, push our trailing newline.
      push @new, "";

      ###  Splice out the old and splice in the new.
      splice @$lref, $OFFSET, $LENGTH, @new;
      &flush_file_lines();

      ### Restart networking.
      system "/etc/init.d/network restart > /dev/null 2>&1";
    }
  }

  ###
  ###  Case 3:  CREATE, DHCP
  ###  
  if ($in{'create'} && ($in{'type'} eq "dhcp")) {
    my ($lref, $if_ID);

    ###  Enable/Disable clobbering of /etc/resolv.conf.
    &ToggleResolvClobbering($in{'clobber_resolv_dhcp'});

    ###  Setup local variables.
    $lref   =  &read_file_lines($INTERFACES_FILE);
    $if_ID  =  GetNextPhysicalInterfaceNumber();

      ###  Push our new entry into the file.
      push @$lref, "iface eth$if_ID inet dhcp\n";

    ###  Finally, write the file to disk and bring the interface up.
    &flush_file_lines();
    &IfUp("eth$if_ID");
  }


  ###
  ###  Case 4:  EDIT/DELETE, DHCP
  ###  
  elsif (($in{'save'} || $in{'delete'}) && ($in{'type'} eq "dhcp")) {
    my ($CURRENT, $lref, $OFFSET, $LENGTH);

    ###  Enable/Disable clobbering of /etc/resolv.conf.
    &ToggleResolvClobbering($in{'clobber_resolv_dhcp'});

    ###  Get $CURRENT.
    foreach my $i (&ReadInterfaces) {
      if ($i->{'interface'} eq $in{'iface'}) { $CURRENT = $i; }
    }

    ###  Setup local variables.
    $lref   =  &read_file_lines($INTERFACES_FILE);
    $OFFSET =  $CURRENT->{'START'};
    $LENGTH =  $CURRENT->{'END'} - $CURRENT->{'START'} + 1;


    ###  There is nothing to "save", so just delete if we are told to.
    if ($in{'delete'}) {
      splice @$lref, $OFFSET, $LENGTH;
      &IfDown($CURRENT->{'interface'});
      &flush_file_lines();
    }

    ###  Save...
    elsif ($in{'save'}) {
      splice @$lref, $OFFSET, $LENGTH, "iface $CURRENT->{'interface'} inet dhcp";
      &IfUp($CURRENT->{'interface'});
      &flush_file_lines();
    }
  }

  ###
  ###  Case 5:  CREATE, PPPOE
  ###  
  elsif ($in{'create'} && ($in{'type'} eq "pppoe")) {
    my ($lref, $if_ID);

    ###  Setup local variables.
    $if_ID  =  GetNextPhysicalInterfaceNumber();

    ###  Save our PPPoE configuration.
    &SavePPPoEConfig("eth$if_ID",           $in{'pppoe_acname'},
                     $in{'pppoe_password'}, $in{'pppoe_servicename'},
                     $in{'pppoe_username'}, $in{'clobber_resolv_pppoe'});

    ###  Read file.
    $lref   =  &read_file_lines($INTERFACES_FILE);

      ###  Push our new entry into the file.
      push @$lref, "iface eth$if_ID inet pppoe\n";

    ###  Finally, write the file to disk and bring the interface up.
    &flush_file_lines();
    &IfUp("eth$if_ID");
  }

  ###
  ###  Case 6:  EDIT/DELETE, PPPOE
  ###  
  elsif (($in{'save'} || $in{'delete'}) && ($in{'type'} eq "pppoe")) {
    my ($CURRENT, $lref, $OFFSET, $LENGTH);

    ###  Get $CURRENT.
    foreach my $i (&ReadInterfaces) {
      if ($i->{'interface'} eq $in{'iface'}) { $CURRENT = $i; }
    }


    ###  Save our PPPoE configuration.
    &SavePPPoEConfig($CURRENT->{'interface'}, $in{'pppoe_acname'},
                     $in{'pppoe_password'},   $in{'pppoe_servicename'},
                     $in{'pppoe_username'},   $in{'clobber_resolv_pppoe'});


    ###  Setup local variables.
    $lref   =  &read_file_lines($INTERFACES_FILE);
    $OFFSET =  $CURRENT->{'START'};
    $LENGTH =  $CURRENT->{'END'} - $CURRENT->{'START'} + 1;


    ###  Delete...
    if ($in{'delete'}) {
      splice @$lref, $OFFSET, $LENGTH;
      &IfDown($CURRENT->{'interface'});
      &flush_file_lines();
    }

    ###  Save...
    elsif ($in{'save'}) {
      splice @$lref, $OFFSET, $LENGTH, "iface $CURRENT->{'interface'} inet pppoe";
      &IfUp($CURRENT->{'interface'});
      &flush_file_lines();
    }
  }


  ###  All done, let's go back to the index page.
  &redirect("index.cgi");

