#!/usr/bin/perl

=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.                                        
 | $Id: network-lib.pl,v 1.1.1.1 2001/12/11 06:50:40 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  do "../../web-lib.pl";
  &init_config();

  ### ##########################################################################
  ### Global Variables
  ### ##########################################################################
  $INTERFACES_FILE  =  '/etc/network/interfaces';
  $RESOLV_FILE      =  '/etc/resolv.conf';
  $POSTFIX_RESOLV   =  '/var/spool/postfix/etc/resolv.conf';
  $HOSTS_FILE       =  '/etc/hosts';
  $POSTFIX_HOSTS    =  '/var/spool/postfix/etc/hosts';
  $PUMP_CONF        =  '/etc/pump.conf';
  $PPPOE_CONF       =  '/etc/ppp/pppoe.conf';
  $PAP_CONF         =  '/etc/ppp/pap-secrets';
  $IPCALC_BIN       =  '/bin/ipcalc';
  $IFUP_BIN         =  '/sbin/ifup';
  $IFDOWN_BIN       =  '/sbin/ifdown';

  %INTERFACE_TYPES  =  ( 'loopback', 'Local Loopback',
                         'static',   'Static Address',
                         'dhcp',     'Dynamic Address (DHCP)',
                         'pppoe',    'Dynamic Address (PPPoE)' );

  @DNSSERVER_ENUMERATION = ("Primary Nameserver",
                            "Secondary Nameserver",
                            "Ternary Nameserver");

  ### ##########################################################################
  sub ReadInterfaces() {
    my (@I, $IN_IFACE, $cnt, $lineno);

    $IN_IFACE = 0;
    $cnt      = -1;
    $lineno   = -1;

    open INTERFACES, $INTERFACES_FILE;
      while (my $line = <INTERFACES>)
      {
        chomp $line;  $lineno++;

        if ($line =~ m/^iface\ (.*)\ inet\ (.*)$/) {
          $cnt++; $IN_IFACE = 1;

          $I[$cnt]->{'interface'} = $1;
          $I[$cnt]->{'type'}      = $2;
          $I[$cnt]->{'START'}     = $lineno;
          if ($1 =~ m/\:/) { $I[$cnt]->{'virtual'} = 1; }
          else             { $I[$cnt]->{'virtual'} = 0; }
        }

        elsif ($IN_IFACE && ($line =~ m/^\s+address\ (.*)/)) {
          $I[$cnt]->{'address'} = $1;
        }
        elsif ($IN_IFACE && ($line =~ m/^\s+netmask\ (.*)/)) {
          $I[$cnt]->{'netmask'} = $1;
        }
        elsif ($IN_IFACE && ($line =~ m/^\s+network\ (.*)/)) {
          $I[$cnt]->{'network'} = $1;
        }
        elsif ($IN_IFACE && ($line =~ m/^\s+broadcast\ (.*)/)) {
          $I[$cnt]->{'broadcast'} = $1;
        }
        elsif ($IN_IFACE && ($line =~ m/^\s+up\ route\ add\ default\ gw\ (.*)/)) {
          $I[$cnt]->{'DEFAULTROUTE'}      = $1;
          $I[$cnt]->{'DEFAULTROUTE_LINE'} = $lineno;
        }
        elsif ($IN_IFACE && ($line eq "")) {
          $I[$cnt]->{'END'} = $lineno;  $IN_IFACE = 0;
        }
      }
    close INTERFACES;

    $I[$cnt]->{'END'} = $lineno;

    return @I;
  }


  ### ##########################################################################
  sub GetRealInterfaces() {
    my (@A, $IN_IFACE, $IS_VIRT, $tmp_iface);

    $IN_IFACE   =  0;
    $IS_VIRT    =  0;
    $tmp_iface  =  "";

    open IFCONFIG, "/sbin/ifconfig -a |";
      while (<IFCONFIG>)
      {
        chomp $_;

        if ($_ =~ m/^(eth[0-9\:]+)\s+/) {
          $IN_IFACE  =  1; $tmp_iface = $1;
          $IS_VIRT   =  1 if ($tmp_iface =~ m/\:/);
        }
        elsif ($_ eq "")  { $IN_IFACE = 0; $IS_VIRT = 0; $tmp_iface = ""; }

        if (($_ =~ m/^\s+inet\ addr\:([0-9\.]+)/) && $IN_IFACE) {
          push @A, { 'iface'   => $tmp_iface,
                     'address' => $1,
                     'virtual' => $IS_VIRT };
        }
    }
      close IFCONFIG;

    return @A;
  }


  ### ##########################################################################
  sub GetNextVirtualInterfaceNumber() {
    my ($if, $rv); $if = $_[0]; $rv = "";

    foreach my $i (&ReadInterfaces())
    {
      if ($i->{'interface'} =~ m/^$if\:([0-9]+)$/) {
        if ($1 > $rv) { $rv = $1; }
      }
    }

    return ++$rv;
  }


  ### ##########################################################################
  sub GetNextPhysicalInterfaceNumber() {
    my ($rv); $rv = "";

    foreach my $i (&ReadInterfaces())
    {
      next if ($i->{'virtual'});

      if ($i->{'interface'} =~ m/^eth([0-9]+)$/) {
        if ($1 > $rv) { $rv = $1; }
      }
    }

    return ++$rv;
  }


  ### ##########################################################################
  sub ListStaticRoutes() {
    my (@R, $lineno, $CURRENT_IFACE);

    $lineno        = -1;
    $CURRENT_IFACE = undef;

    open INTERFACES, $INTERFACES_FILE;
      while (my $line = <INTERFACES>)
      {
        chomp $line; $lineno++;

        ###  Our control variable.
        if ($line eq "")                     { $CURRENT_IFACE = undef; }
        elsif ($line =~ m/^iface\ ([^\ ]+)/) { $CURRENT_IFACE = $1; }

        if (($CURRENT_IFACE ne "lo") && ($line =~ m/^\s*up\ route\ add\ \-net\ ([0-9\.]+)\ netmask\ ([0-9\.]+)\ dev\ (.+)$/)) {
          push @R, { 'network' => $1, 'netmask' => $2,
                     'device'  => $3, 'lineno' => $lineno };
        }

      }
    close INTERFACES;

    return @R;
  }


  ### ##########################################################################
  sub GetRouteRef() {
    my ($rv);  $rv = undef;

    foreach my $r (&ListStaticRoutes()) {
      if ($_[0] eq $r->{'lineno'}) { $rv = $r; }
    }

    return $rv;
  }


  ### ##########################################################################
  sub GetNetwork() {
    my ($rv); $rv = "";
      ($rv) = (`$IPCALC_BIN --network $_[0] $_[1]` =~ m/^NETWORK\=(.*)$/);  
    return $rv;
  }


  ### ##########################################################################
  sub GetBroadcast() {
    my ($rv); $rv = "";
      ($rv) = (`$IPCALC_BIN --broadcast $_[0] $_[1]` =~ m/^BROADCAST\=(.*)$/);  
    return $rv;
  }


  ### ##########################################################################
  sub GetDNSServers() {
    my (@D);

    open RESOLV, $RESOLV_FILE;
      while (my $line = <RESOLV>)
      {
        chomp $line; $lineno++;

        if ($line =~ m/^nameserver\ (.*)$/) {
          push @D, $1;
        }

      }
    close RESOLV;

    return @D;
  }


  ### ##########################################################################
  sub GetSearchDomains() {
    my (@S);

    open RESOLV, $RESOLV_FILE;
      while (my $line = <RESOLV>)
      {
        chomp $line; $lineno++;

        if ($line =~ m/^search\ (.*)$/) {
          return $1;
        }

      }
    close RESOLV;

    return "";
  }


  ### ##########################################################################
  sub ListHosts() {
    my (@H, $lineno);  $lineno = -1;

    open HOSTS, $HOSTS_FILE;
      while (my $line = <HOSTS>)
      {
        my ($k, $v); chomp $line; $lineno++;

        next if (($line eq "") || ($line =~ m/^#/));

        ($k, $v) = split /\s+/, $line, 2;

        push @H, { 'address' => $k, 'hosts' => $v, 'lineno' => $lineno };
      }
    close HOSTS;

    return @H;
  }


  ### ##########################################################################
  sub IsDHCPClobberingResolv() {

    open PUMPCONF, $PUMP_CONF;
      while (my $line = <PUMPCONF>) {
        chomp $line; return 0 if ($line eq "nodns");
      }
    close PUMPCONF;

    return 1;
  }


  ### ##########################################################################
  sub IsPPPoEClobberingResolv() {

    open PPPOECONF, $PPPOE_CONF;
      while (my $line = <PPPOECONF>) {
        chomp $line;

        if ($line =~ m/^DNSTYPE\=(.*)$/) {
          if (($1 eq "SERVER") || ($1 eq "SPECIFY")) { return 1; }
          else                                       { return 0; }
        }

      }
    close PPPOECONF;

    return 1;
  }


  ### ##########################################################################
  sub SavePPPoEConfig() {
    my ($lref);

    ###  Update /etc/ppp/pppoe.conf.
    $lref = &read_file_lines($PPPOE_CONF);

      for (my $i = 0; $i < @$lref; $i++)
      {
        if (@$lref[$i] =~ m/^ETH=/)       { @$lref[$i] = "ETH='$_[0]'"; }
        elsif (@$lref[$i] =~ m/^USER=/)   { @$lref[$i] = "USER='$_[4]'"; }
        elsif (@$lref[$i] =~ m/^ACNAME=/) { @$lref[$i] = "ACNAME='$_[1]'"; }
        elsif (@$lref[$i] =~ m/^SERVICENAME=/) {
          @$lref[$i] = "SERVICENAME='$_[3]'";
        }
        elsif (@$lref[$i] =~ m/^DNSTYPE=/) {
          if ($_[5] eq "yes") { @$lref[$i] = "DNSTYPE=SERVER"; }
          else                { @$lref[$i] = "DNSTYPE=NOCHANGE"; }
        }
    }

    &flush_file_lines();


    ###  Update /etc/ppp/pap-secrets.
    open PAP, ">$PAP_CONF";
      print PAP "# Secrets for authentication using PAP\n";
      print PAP "# client	server	secret			IP addresses\n";
      print PAP "\"$_[4]\"		*	\"$_[2]\"\n";
    close PAP;
    chmod 0600, $PAP_CONF;
  }


  ### ##########################################################################
  sub ToggleResolvClobbering() {

    open PUMPCONF, ">$PUMP_CONF";
      print PUMPCONF "nodns\n" if ($_[0] eq "no");
    close PUMPCONF;
  }


  ### ##########################################################################
  sub isIP() {
    if ($_[0] =~ (m/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/)) {
      return 1;
    }

    return 0;
  }


  ### ##########################################################################
  sub GetPPPoEConfig() {
    my (%P, $lineno);

    ###  Read /etc/ppp/pppoe.conf
    open CONF, $PPPOE_CONF;
      while (my $line = <CONF>)
      {
        my ($k, $v); chomp $line;
        next if (($line eq "") || ($line =~ m/^#/));

        ###  Split, strip quotes, and insert into hash.
        ($k, $v) = split /\=/, $line;
        $v =~ s/^\'//; $v =~ s/^\"//; $v =~ s/\'$//; $v =~ s/\"$//;
        $P{$k} = $v;
      }
    close CONF;

    ###  Read /etc/ppp/pap-secrets
    open CONF, $PAP_CONF;
      while (my $line = <CONF>)
      {
        my (@tmp); chomp $line;
        next if (($line eq "") || ($line =~ m/^#/));

        @tmp = split /\s+/, $line;

        ###  Remove leading/trailing quotes.  Stuff in %P.
        $tmp[2] =~ s/^\"//; $tmp[2] =~ s/\"$//;
        $P{'PASSWORD'}  =  $tmp[2];
      }
    close CONF;

    return %P;
  }


  ### ##########################################################################
  sub GetHostname() {
    my ($rv); $rv = undef;

    $rv = `cat /etc/hostname`; chomp $rv;

    return $rv;
  }


  ### ##########################################################################
  sub SaveHostname() {
    open HOSTNAME, ">/etc/hostname"; print HOSTNAME "$_[0]\n"; close HOSTNAME;

    system("/bin/hostname -F /etc/hostname");
  }


  ### ##########################################################################
  sub IfUp()      { system("$IFUP_BIN $_[0]   > /dev/null 2>&1\n"); }
  sub IfDown()    { system("$IFDOWN_BIN $_[0] > /dev/null 2>&1\n"); }
  sub ifacesort() { $a->{'interface'} cmp $b->{'interface'} };


  ### ##########################################################################
  sub PrintError {
    my ($message) = @_;

    print <<ErrorEOF;
Content-type: text/html


<HTML>
<HEAD>
  <TITLE> </TITLE>

  <SCRIPT LANGUAGE="JavaScript">
    alert("$message");
    history.go(-1);
  </SCRIPT>
</HEAD>
</HTML>
ErrorEOF
  }

  ##############################################################################
  1;
