=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2002 Guardian Digital, Inc., All Rights Reversed
 | $Id: WebTool.pm,v 1.11.2.1 2003/04/10 13:55:54 rwm Exp $
 |
 | Ryan W. Maple <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  # use perl;
  package WebTool;
  use strict;
  use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION $tb $cb %modconfig);

  # Standard module stuff.
  @ISA = ('Exporter');
  @EXPORT = qw($tb $cb &pre %in %modconfig);
  $VERSION = '1.11';    # $Date: 2003/04/10 13:55:54 $

  # List of exportable symbols.  Basically all constants/vars.
  @EXPORT_OK = qw(
			is_valid_ip
			is_valid_subnet
			PROG_MKTEMP
			PROG_OPENSSL
			PROG_TAR
			PROG_ZCAT
			TMPDIR
                 );

  # General constants.
  use constant CONFIGDIR	=> '/etc/webtool';
  use constant PROG_CHKCONFIG	=> '/sbin/chkconfig';
  use constant PROG_MKTEMP	=> '/bin/mktemp';
  use constant PROG_OPENSSL	=> '/usr/bin/openssl';
  use constant PROG_TAIL	=> '/usr/bin/tail';
  use constant PROG_TAR		=> '/bin/tar';
  use constant PROG_ZCAT	=> '/bin/zcat';
  use constant TMPDIR		=> '/tmp/.webtool';

  # UID/GID constants.
  use constant DEFAULT_USERS_GID	=> 100;
  use constant DEFAULT_ADMIN_GID	=> 110;
  use constant LIST_GID_MIN		=> 500;
  use constant LIST_GID_MAX		=> 60000;
  use constant LIST_UID_MIN		=> 500;
  use constant LIST_UID_MAX		=> 60000;

  # Colors.
  $tb = 'BGCOLOR="#717171"';
  $cb = 'BGCOLOR="#F4F4F4"';

  #
  #
  #
  if ($ENV{'SCRIPT_NAME'} && $ENV{'SCRIPT_NAME'} =~ /^\/([^\/]+)\//) {
    my @file = &utilReadFile(CONFIGDIR . "/$1/config");

    foreach my $f (@file) {
      next if ($f =~ m/^\s*\#/);
      my @t = split /\=/, $f, 2;

      $modconfig{$t[0]} = $t[1] if ($t[0]);
    }
  }



################################################################################
sub chkconfig_util($@) {
  my $fcn  = shift;
  my @args = @_;
  my $rv   = undef;

  if ($fcn eq 'list' && $args[0] && $args[1] && $args[1] =~ m/^[0-9]$/) {
    my $cmd = PROG_CHKCONFIG . " --list $args[0]";

    WebTool::output_ctl(0);
    if (open CMD, '-|', $cmd) {
      ($rv) = (<CMD> =~ m/\s+$args[1]\:([ofn]+)/);
      close CMD;
    }
    WebTool::output_ctl(1);
  }

  elsif (($fcn eq 'add' || $fcn eq 'del') && $args[0]) {
    my @cmd = (PROG_CHKCONFIG, "--$fcn", $args[0]);
    WebTool::output_ctl(0); $rv = system @cmd; WebTool::output_ctl(1);
  }

out:
  return $rv;
}

################################################################################
sub basename($) {
  my $name = shift || return undef;
  my @tmp  = split /\//, $name;
  my $len  = @tmp;

  return ($tmp[$len-1]) ? $tmp[$len-1] : undef;
}

################################################################################
sub str_truncate(@) {
  my $string = shift || return undef;
  my $len    = shift || 20;
  my @s      = split //, $string;
  my ($i, $rv);

  for ($i = 0; $i < $len && defined($s[$i]); $i++) { $rv .= $s[$i]; }
  $rv .= '...' unless ($i == length($string));

  return $rv;
}

################################################################################
sub get_user($) {
  my $user = shift || return undef;
  my %rv = ();

  if (my @t = getpwnam($user)) {
    %rv = ( 'username' => $t[0], 'passwd'  => $t[1],
            'uid'      => $t[2], 'gid'     => $t[3],
            'name'     => $t[6], 'homedir' => $t[7],
            'shell'    => $t[8] );
  } 

  return %rv;
}

################################################################################
sub list_users(@) {
  my $arg = shift || 'std';
  my @rv  = ();

  setpwent() or goto out;
  while (my @t = getpwent()) {
    if (($arg eq 'std'     && $t[2] >= 500 && $t[2] < 10000 && $t[6]) ||
        ($arg eq 'winbind'                 && $t[2] > 10000 && $t[6]) ||
        ($arg eq 'all'))
    {
      push @rv, { 'username' => $t[0], 'passwd'  => $t[1],
                  'uid'      => $t[2], 'gid'     => $t[3],
                  'name'     => $t[6], 'homedir' => $t[7],
                  'shell'    => $t[8] };
    }
  }
  endpwent();

out:
  return @rv;
}

################################################################################
sub mktemp(@)  { return mktemp_generic('file',      ($_[0]) ? $_[0] : 'webtool'); }
sub mkdtemp(@) { return mktemp_generic('directory', ($_[0]) ? $_[0] : 'webtool'); }

sub mktemp_generic($$) {
  my $type     = shift || return undef;
  my $template = shift || return undef;
  #my $template = shift || 'webtool';
  my $rv;

  while (1) {
    my @st = lstat(TMPDIR);

    last if (!$st[4] && !$st[5] && $st[2] & 0x4000 && ($st[2] & 0777) == 0700);

    if (@st) { unlink TMPDIR || system('rm', '-rf', TMPDIR); }

    mkdir TMPDIR, 0700 || next;
    chown 0, 0, TMPDIR;
    chmod oct(700), TMPDIR;
  }

  my $cmd;
  if ($type eq 'file') {
    $cmd = PROG_MKTEMP . ' -q ' . TMPDIR . "/$template" . '.XXXXXX';
  }
  elsif ($type eq 'directory') {
    $cmd = PROG_MKTEMP . ' -d -q ' . TMPDIR . "/$template" . '.XXXXXX';
  }

  WebTool::output_ctl(0);
  open MKTEMP, "$cmd |" or return undef; $rv = <MKTEMP>; chomp $rv; close MKTEMP;
  WebTool::output_ctl(1);

  return $rv;
}

################################################################################
sub output_ctl($) {
  my $magic = shift; return undef unless ($magic <= 1 && $magic >= 0);

  if ($magic == 0) {
    open OLDOUT, ">&STDOUT"; open STDOUT, '/dev/null'; select STDOUT; $| = 1;
    open OLDERR, ">&STDERR"; open STDERR, '/dev/null'; select STDOUT; $| = 1;
  }

  elsif ($magic == 1) {
    close STDOUT; open STDOUT, ">&OLDOUT"; select STDOUT; $| = 1; close OLDOUT;
    close STDERR; open STDERR, ">&OLDERR"; select STDOUT; $| = 1; close OLDERR;
  }
}

################################################################################
sub utilReadFile($) {
  my $file = shift || return undef;
  my @rv   = ();
  
  open FILE, $file or return @rv; 
  while (my $l = <FILE>) { chomp $l; push @rv, $l; }
  close FILE;
  
  return @rv;
}

################################################################################
sub utilWriteFile($@) {
  my $file = shift || return undef;
  
  open FILE, '>', $file or return undef;
  foreach my $l (@_) { print FILE "$l\n"; }
  close FILE;
}

################################################################################
sub PrintError($@) {
  my $msg   = shift || return undef;
  my $level = shift || -1;

  print <<EOF;
Content-type: text/html

<HTML>
  <HEAD>
    <TITLE>Error!</TITLE>

    <SCRIPT LANGUAGE="JavaScript">
      alert("$msg");
      history.go($level);
    </SCRIPT>
  </HEAD>
</HTML>
EOF
}

################################################################################
sub Redirect($) {
  my $page = shift || return undef;
  my $url;

  if ($page =~ m/^(http|https)\:/) {
    $url = $page;
  }
  elsif ($page =~ m/^\//) {
    $url  = "https://$ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}$page";
  }
  else {
    #my $tmp = $ENV{'SCRIPT_NAME'}; ($tmp =~ m/^(.*)\/[^\/]+$/);
    #$url= "https://$ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}$tmp/$page";
    $ENV{'SCRIPT_NAME'} =~ m/^(.*)\/[^\/]+$/;
    $url= "https://$ENV{'SERVER_NAME'}:$ENV{'SERVER_PORT'}$1/$page";
  }

  print "Location: $url\n\n";
}

################################################################################
sub Header($$) {
  my $title  = shift || undef;
  my $goback = shift || undef;
  my @tmp    = getSection();

  my $header_img = '/images/SectionHeader-' . $tmp[0] . '.png';
  my $header_url = $tmp[1];

  print <<EOF;
Content-type: text/html

<HTML>
  <HEAD>
    <TITLE>$title</TITLE>

    <LINK REL="STYLESHEET" TYPE="text/css" HREF="/webtool-std.css">
  </HEAD>

  <BODY BGCOLOR="#FFFFFF" LINK="#0000ee" VLINK="#0000ee" TEXT="#000000">
  <TABLE WIDTH="731" BORDER="0">
    <TR>
      <TD WIDTH="25"> &nbsp; </TD>
      <TD WIDTH="682">
        <TABLE BORDER="0" WIDTH="100%">
          <TR>
            <TD ALIGN="CENTER" WIDTH="70%">
              <IMG SRC="$header_img" USEMAP="#header" BORDER="0">

EOF

  #
  #
  #
  if ($header_url ne '/') {
    print <<EOF;
              <MAP NAME="header">
                <AREA SHAPE="RECT" COORDS="2,66,108,91" HREF="$goback">
                <AREA SHAPE="RECT" COORDS="108,66,221,90" HREF="/">
                <AREA SHAPE="RECT" COORDS="221,66,322,90" HREF="/session_login.cgi?logout=1">
                <AREA SHAPE="RECT" COORDS="350,66,679,90" HREF="$header_url">
              </MAP>
EOF
  } else {
    print <<EOF;
              <MAP NAME="header">
                <AREA SHAPE="RECT" COORDS="550,66,679,90" HREF="/session_login.cgi?logout=1">
              </MAP>
EOF
  }

  print <<EOF;
            </TD>
          </TR>
        </TABLE>
<!-- END header -->

EOF

}

################################################################################
sub popup_header($@) {
  my $title  = shift || return undef;
  my $close  = shift || 0;
  my $goback = shift || 0;

  print <<EOF;
Content-type: text/html

<HTML>
  <HEAD>
    <TITLE>$title</TITLE>

    <LINK REL="STYLESHEET" TYPE="text/css" HREF="/webtool-popup.css">
  </HEAD>
  <BODY BGCOLOR="#FFFFFF">
    <TABLE BORDER="0" CELLPADDING="0" CELLSPACING="0" WIDTH="100%">
    <TR>
EOF

  #
  # Print '[ Go Back ]' if we are told to.
  #
  if ($goback) {
    print <<EOF;
      <TD ALIGN="LEFT">
        <B><A HREF="$goback">[&nbsp;Go&nbsp;Back&nbsp;]</A></B>
      </TD>
EOF
  }

  
  #
  # Print '[ Close Window ]' if we are told to.
  #
  if ($close) {
    print <<EOF;
      <TD ALIGN="RIGHT">
        <B><A onClick="window.close()" HREF="#">[&nbsp;Close&nbsp;Window&nbsp;]</A></B>
      </TD>
EOF
  }

  # Close off header.
  print "    </TR>\n";
  print "    </TABLE>\n";
}

################################################################################
sub popup_footer() {
  print "  </BODY>\n";
  print "</HTML>\n";
}

################################################################################
sub close_popup() {
  print "Content-type: text/html\n\n";

  print "<HTML>\n";
  print "  <HEAD>\n";
  print "    <SCRIPT LANGUAGE=\"JavaScript\">\n";
  print "      opener.location.reload();\n";
  print "      window.close();\n";
  print "    </SCRIPT>\n";
  print "  </HEAD>\n";
  print "</HTML>\n";
}

################################################################################
sub Footer() {
  my @links  = ('/GDSN/',
                'http://www.guardiandigital.com/',
                'http://store.guardiandigital.com/html/eng/static/support-webtool.html',
                'http://store.guardiandigital.com',
                '/session_login.cgi?logout=1' );

  print <<EOF;
<!-- BEGIN footer -->
        <TABLE WIDTH="100%" BORDER="0">
          <TR>
            <TD ALIGN="CENTER" VALIGN="CENTER">
              <IMG SRC="/images/WebTool-Footer.png" USEMAP="#footer" BORDER="0">

              <MAP NAME="footer">
                <AREA SHAPE="RECT" COORDS="4,5,303,25" HREF="$links[0]">
                <AREA SHAPE="RECT" COORDS="303,5,443,25" TARGET="__footer2" HREF="$links[1]">
                <AREA SHAPE="RECT" COORDS="443,5,511,25" TARGET="__footer3" HREF="$links[2]">
                <AREA SHAPE="RECT" COORDS="511,5,670,25" TARGET="__footer4" HREF="$links[3]">
              </MAP>
            </TD>
          </TR>
        </TABLE>
      </TD>
    </TR>
  </TABLE>
  </BODY>
</HTML>
EOF
}

################################################################################
sub getSection() {

  if (! $ENV{'REQUEST_URI'}) {
    return ('nomatch', '/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/Security/) {
    return ('Security', '/Security/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/StatusMonitor/) {
    return ('StatusMonitor', '/StatusMonitor/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/SystemBackup/) {
    return ('SystemBackup', '/SystemBackup/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/SystemManager/) {
    return ('SystemManager', '/SystemManager/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/VHost/) {
    return ('VHost', '/VHost/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/GDSN/) {
    return ('GDSN', '/GDSN/');
  }
  elsif ($ENV{'REQUEST_URI'} =~ m/EAS/) {
    return ('EAS', '/EAS/');
  }

  return ('nomatch', '/');

}

################################################################################
sub parseFormData() {
  my (%rv, $in);

  if ($ENV{'REQUEST_METHOD'} eq 'POST') {
    read(STDIN, $in, $ENV{'CONTENT_LENGTH'});
  }
  else { $in = $ENV{'QUERY_STRING'}; }

  my @in = split /\&/, $in;

  foreach my $i (@in) {
    my ($k, $v) = split /=/, $i, 2;

    $k =~ s/\+/ /g; $k =~ s/%(..)/pack("c",hex($1))/ge;
    $v =~ s/\+/ /g; $v =~ s/%(..)/pack("c",hex($1))/ge;

    $rv{$k} = defined($rv{$k}) ? $rv{$k} . "\0$v" : $v;
  }

  return %rv;
}

################################################################################
sub parseFormDataMime() {
  my (%rv, $line, $foo);

  $ENV{CONTENT_TYPE} =~ /boundary=(.*)$/;
  my $boundary = $1;
  my $tmp      = <STDIN>;	# skip first boundary

  while(1) {
    my $name = "";

    # Read section headers
    while(1) {
      $line = <STDIN>;
      chop($line); chop($line);       # lose /r/n

      if (!$line) { last; }
      elsif ($line =~ /^Content-Disposition: form-data(.*)/) {
        my $rest = $1;
        while ($rest =~ /([a-zA-Z]*)=\"([^\"]*)\"(.*)/) {
          if ($1 eq 'name') { $name = $2; }
          else              { $foo = $name . "_$1"; $rv{$foo} = $2; }

          $rest = $3;
        }
      }
      elsif ($line =~ /^Content-Type: (.*)/) {
        $foo = $name . "_content_type";
        $rv{$foo} = $1;
      }
    }

    # Read data
    $rv{$name} .= "\0" if ($rv{$name});
    while(1) {
      $line = <STDIN>;
      if (!$line) { return; }
      if (index($line,"$boundary") != -1) { last; }
      $rv{$name} .= $line;
    }

    chop $rv{$name}; chop $rv{$name};
    if (index($line,"$boundary--") != -1) { last; }
  }

  return %rv;
}

################################################################################
sub mkSubmitTable(@) {
  my $type = shift || return undef;
  my $rv;

  $rv  = "<TABLE BORDER=\"0\" WIDTH=\"100%\" CELLPADDING=\"0\" CELLSPACING=\"0\">\n";
  $rv .= "<TR $cb>\n";

  if ($type == 1) {
    $rv .= "  <TD ALIGN=\"RIGHT\"><DIV CLASS=\"nbody\">\n";
    $rv .= "    <INPUT TYPE=\"SUBMIT\" NAME=\"$_[0]\" VALUE=\"$_[1]\">\n";
    $rv .= "  </DIV></TD>\n";
  } elsif ($type == 2) {
    $rv .= "  <TD ALIGN=\"LEFT\"><DIV CLASS=\"nbody\">\n";
    $rv .= "    <INPUT TYPE=\"SUBMIT\" NAME=\"$_[0]\" VALUE=\"$_[1]\">\n";
    $rv .= "  </DIV></TD>\n";
    $rv .= "  <TD ALIGN=\"RIGHT\"><DIV CLASS=\"nbody\">\n";
    $rv .= "    <INPUT TYPE=\"SUBMIT\" NAME=\"$_[2]\" VALUE=\"$_[3]\">\n";
    $rv .= "  </DIV></TD>\n";
  }

  $rv .= "</TR>\n";
  $rv .= "</TABLE>\n";

  return $rv;
}

################################################################################
sub mkHelpLink($$@) {
  my $display  = shift || return undef;
  my $link     = shift || return undef;
  my $letbreak = shift || 0;
  my $rv;

  $display =~ s/\ /&nbsp;/g unless ($letbreak);

  $rv  = "<DIV CLASS=\"nbody\"><B>";
  $rv .= EnGarde::WebTool::hlink($display, $link);
  $rv .= "</B></DIV>";

  return $rv;
}

################################################################################
sub mkIndexLink($$$@) {
  my $link    = shift || return undef;
  my $title   = shift || return undef;
  my $ditty   = shift || return undef;
  my $disable = shift || 0;
  my $rv;

  $rv  = "      <TR>\n";
  $rv .= "        <TD><DIV CLASS=\"nbody\">\n";
  $rv .= "          <A HREF=\"$link\">" unless ($disable);
  $rv .= "<B>$title</B>";
  $rv .= "</A>\n" unless ($disable);
  $rv .= "          <BR>$ditty\n";
  $rv .= "        </DIV></TD>\n";
  $rv .= "      </TR>\n";
  $rv .= "\n";
  $rv .= "      <TR> <TD> &nbsp; </TD> </TR>\n\n";

  return $rv;
}

################################################################################
sub mkInputCheckbox($$@) {
  my $name    = shift || return undef;
  my $value   = shift;
  my $checked = shift;
  my $rv;

  $rv  = "<INPUT TYPE=\"CHECKBOX\" NAME=\"$name\" VALUE=\"$value\"";
  $rv .= ' CHECKED' if ($value eq $checked);
  $rv .= ">";

  return $rv;
}

################################################################################
sub mkInputRadio($$@) {
  my $name    = shift || return undef;
  my $value   = shift;
  my $checked = shift;
  my $rv;

  $rv  = "<INPUT TYPE=\"RADIO\" NAME=\"$name\" VALUE=\"$value\"";
  $rv .= ' CHECKED' if ($value eq $checked);
  $rv .= ">";

  return $rv;
}

################################################################################
sub mkInputSelect($@) {
  my $name  = shift || return undef;
  my $value = shift;
  my $rv;

  $rv  = "<SELECT NAME=\"$name\">\n";

  for (my $i = 0; defined($_[$i]); $i+=2) {
    $rv .= "  <OPTION VALUE=\"$_[$i]\" ";
    $rv .= "SELECTED" if ($_[$i] eq $value);
    $rv .= ">$_[$i+1]</OPTION>\n";
  }

  $rv .= "</SELECT>\n";

  return $rv;
}

################################################################################
sub mkInputTextArea($@) {
  my $name  = shift || return undef;
  my $value = shift;
  my $rows  = shift || 6;
  my $cols  = shift || 30;
  my $wrap  = shift || 'virtual';
  my $rv;

  $rv  = qq(<TEXTAREA NAME="$name" ROWS="$rows" COLS="$cols" WRAP="$wrap">\n);
  $rv .= "$value";
  $rv .= "</TEXTAREA>\n";

  return $rv;
}

################################################################################
sub mkInputText($$@) {
  my $name      = shift || return undef;
  my $value     = shift || undef;
  my $size      = shift || 30;
  my $maxlength = shift || $size;
  my $rv;

  $rv  = "<INPUT TYPE=\"TEXT\" NAME=\"$name\" VALUE=\"$value\" ";
  $rv .= "SIZE=\"$size\" MAXLENGTH=\"$maxlength\">";

  return $rv;
}

################################################################################
sub mkInputPassword($$@) {
  my $name      = shift || return undef;
  my $value     = shift || undef;
  my $size      = shift || 30;
  my $maxlength = shift || $size;
  my $rv;

  $rv  = "<INPUT TYPE=\"PASSWORD\" NAME=\"$name\" VALUE=\"$value\" ";
  $rv .= "SIZE=\"$size\" MAXLENGTH=\"$maxlength\">";

  return $rv;
}

################################################################################
sub mkPopupLink($$@) {
  my $key     = shift || 'foo';
  my $link    = shift || return undef;
  my $display = shift || return undef;
  my $width   = shift || 350;
  my $height  = shift || 200;
  my $rv;

  $rv  = "<A onClick=\"window.open('$link', '$key', 'toolbar=no,menubar=no,";
  $rv .= "scrollbars=yes,resize=yes,width=$width,height=$height');\" ";
  $rv .= "HREF=\"#\">$display</A>";

  return $rv;
}

################################################################################
sub mkGroupChooserButton($$) {
  my $field = shift || return undef;
  my $multi = shift || 0;
  my $form  = shift || 0;
  my $width = ($multi) ? 500 : 300;
  my $rv;

  $rv  = "<INPUT TYPE=\"BUTTON\" onClick='ifield = document.forms[$form].";
  $rv .= "$field; chooser = window.open(\"/group_chooser.cgi?multi=$multi&";
  $rv .= "group=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no";
  $rv .= "scrollbars=yes,width=$width,height=200\"); chooser.ifield = ifield;'";
  $rv .= " VALUE=\"...\">";

  return $rv;
}

################################################################################
sub mkUserChooserButton($$) {
  my $field = shift || return undef;
  my $multi = shift || 0;
  my $form  = shift || 0;
  my $width = ($multi) ? 500 : 300;
  my $rv;

  $rv  = "<INPUT TYPE=\"BUTTON\" onClick='ifield = document.forms[$form].";
  $rv .= "$field; chooser = window.open(\"/user_chooser.cgi?multi=$multi&";
  $rv .= "user=\"+escape(ifield.value), \"chooser\", \"toolbar=no,menubar=no";
  $rv .= "scrollbars=yes,width=$width,height=200\"); chooser.ifield = ifield;'";
  $rv .= " VALUE=\"...\">";

  return $rv;
}

################################################################################
sub pre() { print "Content-type: text/html\n\n<PRE>\n\n"; }

################################################################################
sub list_interfaces() {
  my (@rv, %state);

  open IFCONFIG, '/sbin/ifconfig 2>/dev/null |' or return ();
  while (my $l = <IFCONFIG>) {
    chomp $l;

    if (! $state{'in'} && $l =~ m/^([A-Za-z0-9]+)\s+Link(.+)$/)  {
      $state{'in'} = 1; $state{'name'} = $1;

      if ($2 =~ m/HWaddr\s+([0-9A-F\:]+)/) { $state{'hwaddr'} = $1; }

      $l = <IFCONFIG>; chomp $l;
      if ($l =~ m/^\s+inet\s+addr\:([0-9\.]+)\s+(.+)/) {
        my $tmp = $2; $state{'addr'} = $1;

        if ($tmp =~ m/Bcast\:([0-9\.]+)\s+/) { $state{'bcast'} = $1; }
        if ($tmp =~ m/Mask\:([0-9\.]+)$/)    { $state{'mask'}  = $1; }
      }
    }

    elsif ($state{'in'} && $l eq '') {
      $state{'in'} = 0;

      $state{'cidr'} = mask2cidr($state{'mask'});
      $state{'net'}  = get_network($state{'addr'}, $state{'mask'});

      push @rv, { 'name'   => $state{'name'}, 'hwaddr' => $state{'hwaddr'},
                  'addr'   => $state{'addr'}, 'bcast'  => $state{'bcast'},
                  'mask'   => $state{'mask'}, 'cidr'   => $state{'cidr'},
                  'net'    => $state{'net'} };
    }
  }
  close IFCONFIG;

  return @rv;
}

################################################################################
sub get_network($$) {
  my $ip   = shift || return undef;
  my $mask = shift || return undef;

  open IPCALC, "/bin/ipcalc --network $ip $mask |" or return ();
  my $rv = <IPCALC>; chomp $rv;
  close IPCALC;

  return ($rv =~ m/^NETWORK\=([0-9\.]+)$/) ? $1 : undef;
}

################################################################################
sub mask2cidr($) {
  my $mask = shift || return undef;
  my $cidr = 0;

  foreach (split(/\./, $mask)) {
    my $bits = unpack("B*", pack("C", $_));
    $cidr += $bits =~ tr/1/1/;
  }

  return $cidr;
}

################################################################################
sub cidr2mask($) {
  my $cidr = shift || return undef;
  my $bits = "1" x $cidr . "0" x (32 - $cidr);

  return join(".", unpack('CCCC', pack("B*", $bits)));
}

################################################################################
sub is_valid_ip($) {
  my $ip = shift || return undef;

  ($ip =~ /^\s*(\d+)\.(\d+)\.(\d+)\.(\d+)\s*$/
    and $1 >= 0 and $1 <= 255
    and $2 >= 0 and $2 <= 255
    and $3 >= 0 and $3 <= 255
    and $4 >= 0 and $4 <= 255);
}

################################################################################
sub is_valid_subnet($) {
  my $mask = shift || return undef;

  if ($mask =~ /^(.+)\/(\d\d?)$/ and is_valid_ip($1) and $2 >= 0 and $2 <= 32) {
    return $1, cidr2mask($2);
  } elsif ($mask =~ /^(.+)\/(.+)$/ and is_valid_ip($1) and is_valid_ip($2)) {
    return $1, $2;
  }

  return;
}

################################################################################
sub to64($$) {
  my $rv;
  my $v      = shift;
  my $n      = shift;
  my @itoa64 = split(//, "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz");

  while(--$n >= 0) { $rv .= $itoa64[$v & 0x3f]; $v >>= 6; }

  return $rv;
}

################################################################################
sub encrypt_passwd($) {
  my $passwd = shift || return undef;
  my $magic =  '$1$';
  my $salt  =  substr(time(), -8);
  my $ctx   =  new Digest::MD5;
  my $j     =  0;

  # Add the password, magic and salt
  $ctx->add($passwd);
  $ctx->add($magic);
  $ctx->add($salt);

  # Add some more stuff from the hash of the password and salt
  my $ctx1 = new Digest::MD5;
  $ctx1->add($passwd);
  $ctx1->add($salt);
  $ctx1->add($passwd);

  # Finally, digest it.
  my $final = $ctx1->digest();

  # Man this md5 stuff is a PITA...
  for (my $pl = length($passwd); $pl > 0; $pl -= 16) {
    $ctx->add($pl > 16 ? $final : substr($final, 0, $pl));
  }

  #
  # This piece of code seems rather pointless, but it's in the C code that
  # does MD5 in PAM so it has to go in!
  #
  for (my $i = length($passwd); $i; $i >>= 1) {
    if ($i & 1) { $ctx->add("\0"); }
    else { $ctx->add(substr($passwd, $j, 1)); }
  }

  # Re-digest.
  $final = $ctx->digest();

  # This loop exists only to waste time
  for (my $i = 0; $i<1000; $i++) {
    $ctx1 = new Digest::MD5;
    $ctx1->add($i & 1 ? $passwd : $final);
    $ctx1->add($salt) if ($i % 3);
    $ctx1->add($passwd) if ($i % 7);
    $ctx1->add($i & 1 ? $final : $passwd);
    $final = $ctx1->digest();
  }

  # Convert the 16-byte final string into a readable form
  my $rv    = $magic.$salt.'$';
  my @final = map { ord($_) } split(//, $final);
  my $l     = ($final[ 0]<<16) + ($final[ 6]<<8) + $final[12];

  $rv .= &to64($l, 4);
  $l = ($final[ 1]<<16) + ($final[ 7]<<8) + $final[13];
  $rv .= &to64($l, 4);
  $l = ($final[ 2]<<16) + ($final[ 8]<<8) + $final[14];
  $rv .= &to64($l, 4);

  $l = ($final[ 3]<<16) + ($final[ 9]<<8) + $final[15];
  $rv .= &to64($l, 4);
  $l = ($final[ 4]<<16) + ($final[10]<<8) + $final[ 5];
  $rv .= &to64($l, 4);
  $l = $final[11];
  $rv .= &to64($l, 2);

  return $rv;
}

################################################################################
1;
